#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)ins_soldistrib.pl 1.8 04/09/30 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package INS_SOLDISTRIB;

use strict;
use variables;

#------------------------------------------------------------------------------
#
# detect the OS version pn the Solaris distrib
#
#------------------------------------------------------------------------------

sub determine_os {

  my ($solaris_dir) = @_;

  foreach my $version (@SUPPORTED_OS) {
    if (-d "$solaris_dir/$version") {
      return $version;
    }
  }

  COMMON::error("The Solaris version on $solaris_dir is unknown");
}

#------------------------------------------------------------------------------
#
# get the release of the Solaris distribution for MEN, diskless and dataless
# and check if it is a supported release
#
#------------------------------------------------------------------------------

sub get_solaris_distrib {

  ($NODE_ID) = @_;

  my $release;

  if (! $USE_SOLARIS_DISTRIB) {
    COMMON::print_stage("No access to any Solaris distribution");
    return;
  }

  if ($CONCERNED_MEN_COUNT > 0) {
    # for MEN
    
    COMMON::print_stage("Retrieving information about Solaris distribution for master-eligible node");
    
    # we get the OS release
    $MEN_TARGET_OS = determine_os($MEN_SOLARIS_DIR);
     $release = COMPONENT::get_release("$MEN_SOLARIS_DIR/$MEN_TARGET_OS/Product/SUNWsolnm/reloc/etc/release");
    
    if (! COMPONENT::check_release($release)) {
      COMMON::error("Unsupported Solaris release ($release)");
    }
    
    SEQUENCER::set_data("MEN_RELEASE", $release);
    
    COMMON::print_action("Solaris release detected: $release");
  }

  # for diskless
  # check only if Solaris distrib will be used
  if ((($INSTALL_TYPE == $CLUSTER_INSTALL) 
       && SEQUENCER::get_data("DISKLESS_ENV"))
      || (($INSTALL_TYPE == $ADD_NODE_INSTALL) 
	  && ($CONCERNED_DISKLESS_COUNT > 0))) {
    COMMON::print_stage("Retrieving information about Solaris distribution for diskless");
    
    # we get the OS release
    $DISKLESS_TARGET_OS = determine_os($DISKLESS_SOLARIS_DIR);
    $release = COMPONENT::get_release("$DISKLESS_SOLARIS_DIR/$DISKLESS_TARGET_OS/Product/SUNWsolnm/reloc/etc/release");
    
    if (! COMPONENT::check_release($release)) {
      COMMON::error("Unsupported Solaris release ($release)");
    }
    
    SEQUENCER::set_data("DISKLESS_RELEASE", $release);
    
    COMMON::print_action("Solaris release detected: $release");
  }

  # for dataless
  # check only if SOlaris distrib will be used
  if ($CONCERNED_DATALESS_COUNT > 0) {

    COMMON::print_stage("Retrieving information about Solaris distribution for dataless");
    
    # we get the OS release
    $DATALESS_TARGET_OS = determine_os($DATALESS_SOLARIS_DIR);
    $release = COMPONENT::get_release("$DATALESS_SOLARIS_DIR/$DATALESS_TARGET_OS/Product/SUNWsolnm/reloc/etc/release");
    
    if (! COMPONENT::check_release($release)) {
      COMMON::error("Unsupported Solaris release ($release)");
    }
    
    SEQUENCER::set_data("DATALESS_RELEASE", $release);

    COMMON::print_action("Solaris release detected: $release");
  }

}

#------------------------------------------------------------------------------
#
# checking of the Solaris distribution to control for MEN
# if it matches the expected one
#
#------------------------------------------------------------------------------

sub check_solaris_distrib {

  ($NODE_ID) = @_;

  if ((! $INSTALL_SOLARIS_ON_MEN) || ($CONCERNED_MEN_COUNT <= 0)) {
    return ;
  }

  COMMON::print_stage("Solaris distribution checking for MEN");

  $MEN_TARGET_OS = determine_os($MEN_SOLARIS_DIR);

  #
  # we check the OS release
  #
  COMMON::print_action("Solaris release checking");

  my $release = COMPONENT::get_release("$MEN_SOLARIS_DIR/$MEN_TARGET_OS/Product/SUNWsolnm/reloc/etc/release");

  my $current_release = SEQUENCER::get_data("MEN_RELEASE");

  if ($release ne $current_release) {
    COMMON::error("Release of the Solaris distribution ($release) is not the expected release ($current_release)");
  }
}

{
}






