#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)local_check.pl 1.11 04/09/30 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package LOCAL_CHECK;

use strict;
use variables;

#------------------------------------------------------------------------------
#
# retrieve the Solaris version
#
#------------------------------------------------------------------------------

sub get_solaris {

  my ($node_ip) = @_;

  my $tmpfile   = "$WORKING_DIR/release.tmp";
  COMMON::get_file($node_ip, "/etc/release", $tmpfile);
 
  my $release = COMPONENT::get_release($tmpfile);

  unlink($tmpfile);

  return $release;

}

#------------------------------------------------------------------------------
#
# retrieve the Solaris version
#
#------------------------------------------------------------------------------

sub local_get_solaris {

  ($NODE_ID) = @_;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $node_name = COMMON::get_external_name($NODE_ID, "");

  my $node_idx = COMMON::getnode($NODE_ID);
  if ($NODE_LIST[$node_idx]{type} == $MEN_NODE) {
    COMMON::print_stage_node("Retrieving information about Solaris");
  } else {
    COMMON::print_stage_node("Retrieving information about Solaris from dataless");
  }

  my $release = get_solaris($node_ip);

  if (! COMPONENT::check_release($release)) {
    COMMON::error("non supported OS release on $node_name ($release)") ;
  }

  if ($NODE_LIST[$node_idx]{type} == $MEN_NODE) {
    SEQUENCER::set_data("MEN_RELEASE", $release);
  } else {
    SEQUENCER::set_data("DATALESS_RELEASE", $release);
  }

  COMMON::print_action("Release detected: $release");
}

#------------------------------------------------------------------------------
#
# checking of Solaris version
#
#------------------------------------------------------------------------------

sub local_check_solaris {

  ($NODE_ID) = @_;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $node_name = COMMON::get_external_name($NODE_ID, "");

  my $current_release;

  my $node_idx = COMMON::getnode($NODE_ID);

  if ($NODE_LIST[$node_idx]{type} == $MEN_NODE) {
    COMMON::print_stage_node("Solaris release checking");
    $current_release = SEQUENCER::get_data("MEN_RELEASE");
  } else {
    COMMON::print_stage_node("Solaris release checking on dataless");
    $current_release = SEQUENCER::get_data("DATALESS_RELEASE");
  }

  my $release = get_solaris($node_ip);

  if ($release ne $current_release) {
    COMMON::error("OS release on $node_name ($release) is not the expected release ($current_release)") ;
  }
}

#------------------------------------------------------------------------------
#
# checking if NHAS is already installed (probably an error)
#
#------------------------------------------------------------------------------

sub local_check_nhas {

  ($NODE_ID) = @_;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $result;

  COMMON::print_stage_node("Node checking");
  COMMON::print_action("Checking if Netra HA Suite already installed");

  # the first CMM package is use for checking
  COMMON::print_debug($DEBUG_CMD, "rsh -n $node_ip \"$RPKGINFO $PACKAGE_TO_CHECK\" 2>&1");
  $result = qx/rsh -n $node_ip \"$RPKGINFO $PACKAGE_TO_CHECK\" 2>&1/;
  chomp($result);
  COMMON::print_debug($DEBUG_CMD, "$result");
  if ($result =~ /ERROR: information[.]*/) {
    # package doesn't exist
    return ;
  }

  # package exists
  print "Netra HA Suite is already installed (re-installation may corrupt your cluster)\n";
  my $c="";
  while (($c ne "y") && ($c ne "Y") && ($c ne "n") && ($c ne "N")) {
    print "Do you want to continue the installation anyway [y/n]: ";
    chomp($c = <STDIN>) ;
  }
  if (($c eq "n") || ($c eq "N")) {
    exit ;
  }
}

{
}
