#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)local_end.pl 1.19 04/09/30 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package LOCAL_END;

use strict;
use variables;

#------------------------------------------------------------------------------
#
# . removing of not configured file
# . setting of the CGTP node name
#
#------------------------------------------------------------------------------

sub men_end {

  ($NODE_ID) = @_;
  
  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $config_in_progress = "/etc/opt/SUNWcgha/not_configured";

  COMMON::print_stage_node("Foundation Services post-installation");

  #
  # nodename is changed with CGTP node name
  # can't be defined before because smosservice requires the node name
  # to be linked to an existing IP address: CGTP interface is not
  # defined during installation so it can't be used too soon
  #
  if ($USE_CGTP) {
    my $cgtp_nodename = COMMON::build_node_name($NODE_ID, "CGTP");
    my $local_nodename = "$WORKING_DIR/nodename";
    my $remote_nodename = "/etc/nodename";
    open(NODENAME, ">${local_nodename}") ||
      COMMON::error("Unable to create ${local_nodename}");

    print NODENAME "${cgtp_nodename}\n";
  
    close(NODENAME);
  
    #
    # remote copy nodename
    #
    COMMON::put_file($node_ip, "$local_nodename", "$remote_nodename");
    unlink($local_nodename);
  }
  
  #
  # remove file /etc/opt/SUNWcgha/not_configured
  #
  COMMON::print_action("Removing $config_in_progress");
  COMMON::remote_exec( "$node_ip", "$RRM -f $config_in_progress" ) ||
    COMMON::error ( "Unable to remove $config_in_progress file" );
	
}
#------------------------------------------------------------------------------
#
# . removing of not configured file
#
#------------------------------------------------------------------------------

sub dataless_end {

  ($NODE_ID) = @_;
  
  my $node_ip   = COMMON::get_external_ip($NODE_ID);
  my $config_in_progress = "/etc/opt/SUNWcgha/not_configured";

  my $local_vfstab = "$WORKING_DIR/vfstab";
  my $remote_vfstab = "/etc/vfstab";

  my $master_name;
  my $master_nic0;
  my $master_cgtp;

  COMMON::print_stage_node("Foundation Services post-installation for dataless");

  $master_nic0 = COMMON::build_node_name($MASTER_ID, "NIC0") ;
  $master_name = $master_nic0;
  if ($USE_CGTP) {
    $master_cgtp = COMMON::build_node_name($MASTER_ID, "CGTP") ;
    $master_name = $master_cgtp;
  }
  #
  # retrieve /etc/vfstab file
  #
  COMMON::get_file($node_ip, "$remote_vfstab", "$local_vfstab");

  #
  # add an entry for each remote directory
  #
  for my $i (0..$#DIR_MOUNTED) {
    # create the mounting point
    COMMON::remote_exec($node_ip, "$RMKDIR -p $DIR_MOUNTED[$i]{local}" ) ||
      COMMON::error ( "Unable to create $DIR_MOUNTED[$i]{local}" );
    
    my $option = "rw,hard,fg,intr";
    if ($DIR_MOUNTED[$i]{noac}) {
      $option .= ",noac";
    }
    
    COMMON::update_vfstab_nfs($local_vfstab,
		      "${master_name}:$DIR_MOUNTED[$i]{remote}",
		      "-\t$DIR_MOUNTED[$i]{local}\tnfs\t-\tyes\t$option")
  }
  #
  # putback modified proto_node /etc/vfstab on diskless node branch
  #
  COMMON::put_file($node_ip, "$local_vfstab", "$remote_vfstab");

  
  #
  # remove file /etc/opt/SUNWcgha/not_configured
  #
  COMMON::print_action("Removing $config_in_progress");
  COMMON::remote_exec( "$node_ip", "$RRM -f $config_in_progress" ) ||
    COMMON::error ( "Unable to remove $config_in_progress file" );
	
}

{
}
