#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)sol_jumpstart.pl 1.3 04/10/08 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2001-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package SOL_JUMPSTART;

use strict;
use variables;

require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";

my $SOLARIS_DIST ;
my $NODE_TYPE;
my $JUMPSTART;
my $MOUNTPOINT = "/a/mnt_tmp" ;
my $FINISH_FILENAME;

#------------------------------------------------------------------------------
#
# Work-around for dmfe driver
#
#------------------------------------------------------------------------------

sub dmfe_work_around {

   open (FINISH, ">>$FINISH_FILENAME") || 
     COMMON::error("Unable to update $FINISH_FILENAME file");

   print FINISH <<"FEO";

#--------------------------------------------
# workaround for dmfe driver
#--------------------------------------------

# Bugfix for 4511259
# create the generic device 

DRV="dmfe"
BASEDIR="/a"
grep -w "\${DRV}" \${BASEDIR}/etc/name_to_major > /dev/null 2>&1
if [ \$? -eq 0 ]; then
    echo ""
    echo "--------------------------------------------"
    echo "NHAS installation: "dmfe" driver detected"
    echo "creating device (workaround for bug 4511259)"
    set `/usr/bin/grep "^\${DRV}[  ]" \$BASEDIR/etc/name_to_major`
    DRIVER_MAJ=\$2
    DEVICE=\${BASEDIR}/dev/\${DRV}
    rm -f \$DEVICE   > /dev/null 2>&1
    mknod \$DEVICE c \$DRIVER_MAJ 0 > /dev/null 2>&1
    chmod 600 \$DEVICE
    chgrp sys \$DEVICE
    chown root \$DEVICE
    echo "--------------------------------------------"
    echo ""
fi

FEO

  close(FINISH);

}

#------------------------------------------------------------------------------
#
# SVM installation and configuration
#
#------------------------------------------------------------------------------

sub svm_install {

  my ($solaris_dir) = @_;
  
  # installation of package: there are not located on the standard directory

  my $svm_package_subdir = COMPONENT::get_info_men_component("SVM", "DIR");
  my $svm_packages = COMPONENT::get_info_men_component("SVM", "PACKAGE");

   open (FINISH, ">>$FINISH_FILENAME") || 
     COMMON::error("Unable to update $FINISH_FILENAME file");

   print FINISH <<"FEO";

$RMKDIR -p $MOUNTPOINT

# mount distribution
$RMOUNT $SERVER_IP:/$solaris_dir $MOUNTPOINT

# create admin file
$RECHO "mail=" > /a/tmp/admin
$RECHO "instance=overwrite" >> /a/tmp/admin
$RECHO "partial=nocheck" >> /a/tmp/admin
$RECHO "runlevel=quit" >> /a/tmp/admin
$RECHO "idepend=nocheck" >> /a/tmp/admin
$RECHO "rdepend=nocheck" >> /a/tmp/admin
$RECHO "space=quit" >> /a/tmp/admin
$RECHO "setuid=nocheck" >> /a/tmp/admin
$RECHO "conflict=nocheck" >> /a/tmp/admin
$RECHO "action=nocheck" >> /a/tmp/admin
$RECHO "basedir=default" >> /a/tmp/admin

# install packages
$RPKGADD -d $MOUNTPOINT/$svm_package_subdir -R /a -a /a/tmp/admin $svm_packages
# umount distribution
$RUMOUNT $MOUNTPOINT
$RRMDIR $MOUNTPOINT

FEO

  close(FINISH);

}

#------------------------------------------------------------------------------
#
#  Additional packages and patches installation
#
#------------------------------------------------------------------------------

sub addon_install {

  
  # installation of package: there are not located on the standard directory

  ADDON::update_finish ($NODE_TYPE, $FINISH_FILENAME, $MOUNTPOINT);

}

#------------------------------------------------------------------------------
#
# Create jumpstart environment
#
#------------------------------------------------------------------------------
sub jumpstart_create {

  my ($solaris_dir, $profile_file) = @_;

   my $proto_name  = COMMON::build_node_name_for_install($NODE_ID);
   my $server_name = COMMON::build_server_name();
   my $proto_ip    = COMMON::get_external_ip($NODE_ID);
   my $ether       = $NODE_LIST[COMMON::getnode($NODE_ID)]{eth0};

   my $cmd;
   
   #
   # Controle the host configuration
   #

   #
   # Test the  jumpstart directory tree, create it if it does not exist
   #

   COMMON::control_dir_rxw_or_create ( "$JUMPSTART" );
   COMMON::control_dir_rxw_or_create ( "$JUMPSTART/$CLUSTER" );
   COMMON::control_dir_rxw_or_create ( "$JUMPSTART/$CLUSTER/$NODE_ID" );

   $FINISH_FILENAME = "$JUMPSTART/$CLUSTER/$NODE_ID/finish.sh";
   #
   # Create Jumpstart config files
   #
   create_sysidcfg ();
   create_profile ($profile_file);
   create_finish();
   create_and_test_rules();   
   
   # 
   # Control and modify /etc/hosts
   #   
   add_host( $proto_ip, $proto_name  );

   # 
   # Control end modify /etc/ethers
   #
   &add_ether ( $ether, $proto_name );
    
   #
   # Purge preexistant boot client, ignore errors silently
   #
   COMMON::exec_cmd_ignore ("$SOLARIS_DIST/Tools/rm_install_client $proto_name > /dev/null 2>&1") ;
   
   #
   # Share
   #        Jumpstart environment
   COMMON::share("$JUMPSTART" );
      
   #
   # Add boot client
   #
   $cmd="$SOLARIS_DIST/Tools/add_install_client";
   $cmd .= " -i \"$proto_ip\"";
   $cmd .= " -e \"$ether\"";
   $cmd .= " -s \"$SERVER_IP:$solaris_dir\"";
   $cmd .= " -c \"$SERVER_IP:${JUMPSTART}/${CLUSTER}/${NODE_ID}\"";
   $cmd .= " -p \"$SERVER_IP:${JUMPSTART}/${CLUSTER}/${NODE_ID}\"";
   $cmd .= " -n :none";
   $cmd .= " $proto_name $ARCH";   
   COMMON::exec_cmd ($cmd) ||
     COMMON::error ("Execution of \"$cmd\" failed"); 
   
   #
   # Remplace install server name by server IP address in /etc/bootparams
   #
   $cmd="/usr/bin/perl -p -i.bak";
   $cmd .= " -e 'if ( /^$proto_name/ ) {";
   $cmd .= " s/^$proto_name\\s+[^:]+:/$proto_name root=$SERVER_IP:/;";
   $cmd .= " }'";
   $cmd .= " /etc/bootparams";
   COMMON::exec_cmd ($cmd) ||
     COMMON::error("Can't modify \"/etc/bootparams\" file");
      
}


#------------------------------------------------------------------------------
#
# Management of /etc/hosts
#
#------------------------------------------------------------------------------

sub add_host {
   my ( $address, $name ) = @_ ;
   my $count=0;
   my $ip;
   my $host;
   my $garbage;
 
   $count = 0;   
     
   open (HOSTS, "/etc/hosts" ) || &error("Unable to open /etc/hosts");
   while ( <HOSTS> ) {
      
      chop();
      s/#.*//;
      ($ip, $host, $garbage) = split (/[\s\t]+/, $_) ;
      next unless (defined($ip));
      next unless (defined($host));
            
      if ( "$ip" eq "$address" ) {
	 COMMON::error("Ip \"$address\" does not match \"$name\" as first hostname in /etc/hosts") 
	 unless ( $host eq $name );
         $count++;
	 last;
      }
       	        
   }
   close (HOSTS);
   
   unless ( $count ) {
      open (HOSTS, ">>/etc/hosts" ) || COMMON::error("Unable to update /etc/hosts");
      print HOSTS "$address $name\n";
      close (HOSTS);
   }
   
}

#------------------------------------------------------------------------------
#
# Management of /etc/ethers files
#
#------------------------------------------------------------------------------
sub add_ether {
   
   my ( $address, $name ) = @_ ;
   my $count=0;
   my $ether;
   my $host;
      
   if ( -f "/etc/ethers" ) {
      open (ETHERS, "/etc/ethers" ) || COMMON::error("Unable to open /etc/ethers");
      while ( <ETHERS> ) {
      
         chop();
         s/#.*//;
         ($ether, $host) = split (/[\s\t]+/, $_) ;
         next unless (defined($ether));
         next unless (defined($host));
      
         if ( ethermatch ("$ether","$address") ) {  
	    COMMON::error("MAC \"$address\" does not match \"$name\" as hostname in /etc/ethers")
	    unless ( $host eq $name );
            $count++;
	    last;
         }
       	        
      }
      close (ETHERS);
   }
   
   unless ( $count ) {
      open (ETHERS, ">>/etc/ethers" ) || COMMON::error("Unable to update /etc/ethers");
      print ETHERS "$address $name\n";
      close (ETHERS);

      COMMON::exec_cmd("$PKILL in.rarpd ; /usr/sbin/in.rarpd -a") ||
   	COMMON::error("Unable to kill and restart rarpd daemon file");

   }

}


sub duplicated_ether {
   
   my ( $address ) = @_ ;
   my $count=0;
   my $ether;
   my $host;
         
   if ( -f "/etc/ethers" ) {
      open (ETHERS, "/etc/ethers" ) || COMMON::error("Unable to open /etc/ethers");
      while ( <ETHERS> ) {      
         chop();
         s/#.*//;
         ($ether, $host) = split (/[\s\t]+/, $_) ;
         next unless (defined($ether));
         next unless (defined($host));      
         $count++ if ( ethermatch($ether,$address) );         	        
      }
      close (ETHERS);
   }

   return ( $count > 1 );
      
}

sub ethermatch {

   my ( $addr1, $addr2 ) = @_;

   my $i;

   
   return 0 unless (defined($addr1));
   return 0 unless (defined($addr2));
   
   my @a = split ( /:/, $addr1 );
   my @b = split ( /:/, $addr2 );
   
   for ( $i=0; $i < 6; $i++ ) {
      return 0 unless (hex($a[$i]) == hex($b[$i])); 
   }
   
   return 1;
      
}

#------------------------------------------------------------------------------
#
# Create Sysidcfg file
#
#------------------------------------------------------------------------------
sub create_sysidcfg {

   my $node_ip   = COMMON::get_external_ip($NODE_ID);
   my $node_name = COMMON::build_node_name_for_install($NODE_ID);

   my $timezone = SEQUENCER::get_data("TIMEZONE");
    
   open (SYSIDCFG, ">$JUMPSTART/$CLUSTER/$NODE_ID/sysidcfg") || 
     COMMON::error("Unable to create \"$JUMPSTART/$CLUSTER/$NODE_ID/sysidcfg\" file");
   print SYSIDCFG <<"FEO";
name_service=NONE
system_locale=$LOCALE
timezone=$timezone
timeserver=$SERVER_IP
security_policy=NONE
terminal=$TERMINAL
network_interface=primary {protocol_ipv6=no
                           hostname=$node_name
                           ip_address=$node_ip
		      	   default_route=$DEFAULT_ROUTER_IP
                           netmask=$DOTTED_NETMASK}
root_password=$ENCRYPTED_PASSWORD
FEO

   close(SYSIDCFG);

}

#------------------------------------------------------------------------------
#
# Create jumpstart profile file
#
#------------------------------------------------------------------------------
sub create_profile {

  my ($profile_file) = @_;

  my $profile = "$JUMPSTART/$CLUSTER/$NODE_ID/node.prof" ;

  my $slice_root ;
  my $node_idx = COMMON::getnode($NODE_ID);
  my $slice_idx;
  my $mounting_point;

  COMMON::exec_cmd( "$CP $profile_file $profile") ||
      COMMON::error("Unable to create $profile file");

  COMMON::slice_search($NODE_ID, "mounting_point", $SLICE_ROOT, \$slice_idx ) ;
  $slice_root= $SLICE_LIST[$slice_idx]{name};

  open(FPROFILE, ">>$profile") ;

  print FPROFILE "boot_device $slice_root update\n" ;
  print FPROFILE "root_device $slice_root\n" ;

  for $slice_idx (@{$NODE_LIST[$node_idx]{slice}}) {
    # particular case: "replica" must be replaced with "unnamed"
    $mounting_point = $SLICE_LIST[$slice_idx]{mounting_point} ;
    if ($mounting_point eq $SLICE_TAG_REPLICA) {
      $mounting_point = $SLICE_UNMOUNTED;
    }

    print FPROFILE "filesys $SLICE_LIST[$slice_idx]{name} $SLICE_LIST[$slice_idx]{size} $mounting_point $SLICE_LIST[$slice_idx]{option}\n"
  }
  
  #
  # generate a "usedisk" directive for each disk present on the node
  #
  my %disk_array;
  for $slice_idx (@{$NODE_LIST[$node_idx]{slice}}) {
    my $disk = substr($SLICE_LIST[$slice_idx]{name}, 0, 6);
    if (! exists $disk_array{$disk}) {
      print FPROFILE "usedisk $disk\n" ;
      $disk_array{$disk} = "";
    }
  }

  close FPROFILE ;

}

#----------------------------------------------------------------------------- 
#
# Create jumpstart rules file
#
#------------------------------------------------------------------------------
sub create_and_test_rules {
  
   my $output = "";

   open (RULES, ">$JUMPSTART/$CLUSTER/$NODE_ID/rules") || 
     COMMON::error("Unable to create \"$JUMPSTART/$CLUSTER/$NODE_ID/rules\" file");
   my $node_ip = COMMON::get_external_ip($NODE_ID);
   print RULES "hostaddress $node_ip - node.prof finish.sh\n";
   close (RULES);

   open (CHECK,"cd $JUMPSTART/$CLUSTER/$NODE_ID; $SOLARIS_DIST/Misc/jumpstart_sample/check |") || 
     COMMON::error ("Unable to check \"rules\" file"); 
   while ( <CHECK> ) {
     chop();
     COMMON::print_debug($DEBUG_FILE, $_);
     $output= "$output $_\n";
   }
   if (! close (CHECK)) {
     COMMON::print_log($output);
     COMMON::error("Error found while checking \"Rules\" file");
   }
}

#------------------------------------------------------------------------------
#
# Create jumpstart finish.sh file
#
#------------------------------------------------------------------------------
sub create_finish {

  my $node_nic0;
  my $node_nic1;
  my $node_cgtp;
  my $other_id;
  
  open (FINISH, ">$FINISH_FILENAME") || 
    COMMON::error("Unable to create $FINISH_FILENAME file");
  print FINISH "#!/bin/sh\n\n";
  
  print FINISH <<"FEO";
#
# create /etc/notrouter
#
if [ ! -f /a/etc/notrouter ] ; then
/usr/bin/touch /a/etc/notrouter
fi

#
# modify /etc/default/login
#
/usr/bin/mv /a/etc/default/login /a/etc/default/login.orig
/usr/bin/chmod 644 /a/etc/default/login.orig
/usr/bin/sed '1,\$s/^CONSOLE/#CONSOLE/' /a/etc/default/login.orig > /a/etc/default/login
/usr/bin/chmod 444 /a/etc/default/login

#
# disable power management
#
if [ ! -f /a/noautoshutdown ] ; then
/usr/bin/touch /a/noautoshutdown
fi


#
# set boot PROM params
#
/usr/sbin/eeprom local-mac-address?=true
/usr/sbin/eeprom auto-boot?=true

FEO

#
# only the install server is listed on /.rhosts in all cases
# moreover the file protection is set up
#
    print FINISH <<"FEO";
#
# modify /.rhosts
#
if [ -f /a/.rhosts ] ; then
/usr/bin/cp /a/.rhosts /a/.rhosts.orig
fi
echo "$SERVER_IP root" >> /a/.rhosts
/usr/bin/chmod 444 /a/.rhosts

FEO

  if (! $RESTRICT_RHOSTS) {

#
# the other MEN (if the node is a MEN) is listed on /.rhosts
#
    
    if ($NODE_TYPE == $MEN_NODE) {

      if ($NODE_ID eq $MEN1_ID) {
	$other_id = $MEN2_ID ;
      } else {
	$other_id = $MEN1_ID ;
      }
      $node_nic0 = COMMON::build_node_name($other_id, "NIC0");

      if ($USE_CGTP) {
	$node_nic1 = COMMON::build_node_name($other_id, "NIC1");
	$node_cgtp = COMMON::build_node_name($other_id, "CGTP");

	print FINISH <<"FEO";
echo "$node_nic0 root" >> /a/.rhosts
echo "$node_nic1 root" >> /a/.rhosts
echo "$node_cgtp root" >> /a/.rhosts

FEO

      } else {
	
	print FINISH <<"FEO";
echo "$node_nic0 root" >> /a/.rhosts

FEO
    
      }
    }
  }
  close (FINISH);
}



#------------------------------------------------------------------------------
#
#  Solaris installation
#
#------------------------------------------------------------------------------

sub prepare_jumpstart {
  
  ($NODE_ID) = @_;
  
  my $solaris_dir;
  my $target_os;
  my $profile_file;
  
  my $node_idx = COMMON::getnode($NODE_ID);
  $NODE_TYPE = $NODE_LIST[$node_idx]{type};
  
  if ($NODE_TYPE == $MEN_NODE) {  

    $solaris_dir  = $MEN_SOLARIS_DIR;
    $target_os    = $MEN_TARGET_OS;
    $profile_file = $MEN_PROFILE;
      
  } else {

    $solaris_dir  = $DATALESS_SOLARIS_DIR;
    $target_os    = $DATALESS_TARGET_OS;
    $profile_file = $DATALESS_PROFILE;

  }
  
  if ($NODE_TYPE == $MEN_NODE) {
    COMMON::print_stage_node("Solaris installation on master-eligible node");
  } else {
    COMMON::print_stage_node("Solaris installation on dataless node");
  }
  
  # Test Solaris distribution directory
  
  COMMON::control_dir_is_readable ("$solaris_dir");
  $SOLARIS_DIST=$solaris_dir . "/$target_os";
  if (! -d $SOLARIS_DIST) {
    COMMON::error("Unable to find $target_os distribution on $solaris_dir") 
  }

  COMMON::control_dir_is_readable ("$SOLARIS_DIST");
  COMMON::control_dir_is_readable ("$SOLARIS_DIST/Tools");
  COMMON::control_dir_is_readable ("$SOLARIS_DIST/Tools/Boot");
  COMMON::control_dir_is_readable ("$SOLARIS_DIST/Misc");
  COMMON::control_dir_is_readable ("$SOLARIS_DIST/Misc/jumpstart_sample");
  
  COMMON::control_file_is_readable ("$SOLARIS_DIST/Misc/jumpstart_sample/check");
  
  COMMON::var_defined("JUMPSTART_DIR", "", \$JUMPSTART);

  COMMON::control_file_is_readable ($profile_file);
  
  COMMON::control_file_is_executable("$SOLARIS_DIST/Tools/add_install_client");
  COMMON::control_file_is_executable("$SOLARIS_DIST/Tools/rm_install_client");
  
  COMMON::control_dir_is_readable ("$SOLARIS_DIST/Tools/Boot/sbin");
  COMMON::control_file_is_executable ("$SOLARIS_DIST/Tools/Boot/sbin/bpgetfile");
    
  # Prepare a jumpstart environment
  COMMON::print_action("Preparing the Jumpstart environment");
  jumpstart_create ($solaris_dir, $profile_file);
  if (($USE_SVM) && ($target_os eq "Solaris_8")) {
    COMMON::print_action("Configuring finish.sh for Solaris Volume Manager (SVM) installation");
    if ($NODE_TYPE == $MEN_NODE) {
      svm_install($solaris_dir);
    }
  }

  # install additional packages or patches during the Solaris installation
  # thru the finish.sh file
  addon_install();

  #
  # work-around for the dmfe driver (which required two reboots to
  # be conrrectly configured
  #
  dmfe_work_around();
}

{
}
