#!/usr/bin/perl
# -----------------------------------------------------------------------------
#  ident "@(#)component.pl 1.7 04/10/08 SMI"
# -----------------------------------------------------------------------------
#
#  Copyright 2003-2004 Sun Microsystems, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

package COMPONENT;

use strict;
use variables;

require "$ENV{NHINSTALL_LIB}/tools/lib/conf.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/common.pl";
require "$ENV{NHINSTALL_LIB}/tools/lib/sequencer.pl";

#==============================================================================
#
#    COMMON ROUTINES
#
#==============================================================================

sub set_package_info {

  my ($scope, $refdir, $refbasedir, $refoptcmd, $refadj) = @_ ;

  #
  #  configure settings depending on location
  #
  $$refdir     = "/opt/SUNWcgha";
  $$refbasedir = "" ;
  $$refoptcmd  = "" ;
  $$refadj     = "" ;

  if ($scope eq $SCOPE_LOCAL) {

    $$refdir     = "/opt/SUNWcgha";
    $$refbasedir = "default" ;
    $$refoptcmd  = "" ;
    $$refadj     = "local" ;

  } elsif ($scope eq $SCOPE_SHARED) {

    $$refdir     = "$SHARED_FS/$OPT_SUBDIR" ;
    $$refbasedir = "/$OPT_SUBDIR" ;
    $$refoptcmd  = "-M -R $SHARED_FS" ;
    $$refadj     = "shared" ;

  } elsif ( $scope eq $SCOPE_USR_SPECIFIC) {

    $$refdir     = "" ;
    $$refbasedir = $SERVICE_USR_BASEDIR;
    $$refoptcmd  = "-M -R $SERVICE_USR_PATH" ;
    $$refadj     = "usr-specific" ;

  } elsif ( $scope eq $SCOPE_USR_SOLARIS) {

    $$refdir     = "" ;
    $$refbasedir = $SERVICE_USR_SOLARIS_BASEDIR;
    $$refoptcmd  = "-M -R $SERVICE_USR_SOLARIS_PATH" ;
    $$refadj     = "usr-root" ;

  } elsif ( $scope eq $SCOPE_CLONE_OPT) {

    $$refdir     = "" ;
    $$refbasedir = $SERVICE_CLONE_OPT_BASEDIR;
    $$refoptcmd  = "-M -R $SERVICE_CLONE_OPT_PATH" ;
    $$refadj     = "clone area" ;

  } else {
    COMMON::error("Invalid option ($scope) for package") ;
  }

}
#==============================================================================
#
#    PACKAGES MANAGEMENT ROUTINES
#
#==============================================================================

#------------------------------------------------------------------------------
#
#  clean up when a package installation fails
#
#  1: admin file
#  2: answers file
#  3: node ip address
#
#------------------------------------------------------------------------------

sub exit_install_pkg {
  
  my ($admin_file, $answers_file, $node_ip, $package, $dir, $subdir) = @_;

  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file) ;
  }
  COMMON::remote_exec ( $node_ip, "$RUMOUNT -f /mnt" );
  COMMON::error("Unable to install packages $package (from $dir/$subdir)" );
}

#------------------------------------------------------------------------------
#
#  remove a package from a node with local disk (errors are ignored)
#
#  1: Node ID
#  2: Package list
#  3: where to install
#
#------------------------------------------------------------------------------

sub remove_local_pkg {

  my ($node_id, $package_list, $scope) = @_;

  my $node_ip = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  my $dir;
  my $basedir;
  my $optcmd;
  my $adj;

  set_package_info ($scope, \$dir,  \$basedir, \$optcmd, \$adj);

  COMMON::print_action("Removing $adj packages: $package_list ...") ;

  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=quit
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # remove the packages in the reverse order
  #
  # do not display error
  my @list = split(" ",$package_list) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
    # remove the ending part of package name like SUNWefcx.u
    # (the package is known as SUNWefcx after installation)
    my $pkg = $list[$i];
    my $pos = index($pkg, ".");
    if ($pos > -1) {
      $pkg = substr($pkg, 0, $pos);
    }
    COMMON::remote_exec_ignore($node_ip, 
			       "echo y | $RPKGRM $optcmd -a /tmp/admin $pkg" );
  }

  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ($node_ip, "$RM /tmp/admin" ) ;

  # remove the local adminfile
  unlink ($admin_file) ;
}

#------------------------------------------------------------------------------
#
# Install a package on a node with localdisk
#
# 1: Node ID
# 2: Package list
# 3: location
# 4: answers file (null string if not required)
#
# if exists, the answers file is deleted before exiting !
#
#------------------------------------------------------------------------------

sub install_local_pkg {

  my ($node_id, $root_dir, $package_subdir, $package_list, 
      $scope, $answers_file) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  my $basedir;
  my $dir;
  my $optcmd;
  my $adj;

  set_package_info($scope, \$dir, \$basedir, \$optcmd, \$adj);
  COMMON::print_action("Installing $adj packages: $package_list ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$package_subdir");
  
  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
  
  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # copy the answers file on the remote node
  #
  if ($answers_file ne "") {
    COMMON::exec_cmd( "$RCP $answers_file $node_ip:/tmp/answers" ) ||
      COMMON::error( "Unable to copy $answers_file on $node_ip:/tmp/answers") ;
  }

  #
  # Remotely create the install directory in case it is needed by BASE_DIR
  #
  if ($dir ne "") {
    COMMON::remote_exec ($node_ip, "$RMKDIR -p $dir" ) ||
      COMMON::error("Unable to create $dir directory needed by BASE_DIR package installation setup");
  }

  #
  # Install packages remotely
  #
  if ($answers_file eq "") {
    COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$package_subdir $optcmd $package_list" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  } else {
    COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -r /tmp/answers -d /mnt/$package_subdir $optcmd $package_list" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }
  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ||
    exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  
  #
  # remove the answers file from the remote node
  #
  if ($answers_file ne "") {
    COMMON::remote_exec ( "$node_ip", "$RM /tmp/answers" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }

  #
  # remove previously created pkgadd admin and answers file
  #
  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) ||
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# Remove packages for all diskless on a MEN
#
# 1: Node ID
# 2: Package list
#
#------------------------------------------------------------------------------

sub remove_diskless_pkg {

  my ($nodeid, $package_list) = @_;

  my $node_ip     = COMMON::get_external_ip($nodeid);
  my $admin_file  = "$WORKING_DIR/admin";
  
  COMMON::print_action("Remove packages for diskless nodes: $package_list ...");

  # 
  # Create a pkgrm admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");
  
  my $basedir="default" ;

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=quit
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;

  #
  # remove packages for all DISKLESS concerned by the installation
  #
  for my $node (0..$#NODE_LIST) {

    if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {

      my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
      my $node_root = "/export/root/$node_name_template";

      COMMON::print_subaction("for $node_name_template ...") ;
      #
      # remove the packages in the reverse order
      #
      # do not display error
      my @list = split(" ",$package_list) ;
      for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
	# remove the ending part of package name like SUNWefcx.u
	# (the package is known as SUNWefcx after installation)
	my $pkg = $list[$i];
	my $pos = index($pkg, ".");
	if ($pos > -1) {
	  $pkg = substr($pkg, 0, $pos);
	}
	COMMON::remote_exec_ignore ( "$node_ip", "echo y | $RPKGRM -a /tmp/admin -R $node_root $pkg" ) ;
      }
    }
  }
  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ;
  
  # remove previously created pkgadd admin file
  #
  unlink($admin_file);	
}

#------------------------------------------------------------------------------
#
# Install a package for all diskless on a MEN
#
# 1: Node ID
# 2: Package list
# 3: answers file (null string if not required)
#
# if exists, the answers file is deleted before exiting !
#
#------------------------------------------------------------------------------

sub install_diskless_pkg {

  my ($node_id, $root_dir, $package_subdir, $package_list, $answers_file) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  
  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$package_subdir");
  
  COMMON::print_action("Installing packages for diskless nodes: $package_list ...") ;

  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
  
  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");
  
  my $basedir="default" ;

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;
  
  #
  # copy the answers file on the remote node
  #
  if ($answers_file ne "") {
    COMMON::exec_cmd( "$RCP $answers_file $node_ip:/tmp/answers" ) ||
	  COMMON::error ( "Unable to copy $answers_file on $node_ip:/tmp/answers") ;
  }

  #
  # Install packages on DISLESS concerned by the installation
  #
  my $node;
  foreach $node (0..$#NODE_LIST) {
    if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {
      my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
      my $node_root = "/export/root/$node_name_template";

      #
      # Remotely create the install directory in case it is needed by BASE_DIR
      #
      COMMON::remote_exec ( "$node_ip", "$RMKDIR -p $node_root/opt/SUNWcgha " ) ||
	COMMON::error("Unable to create $node_root/opt/SUNWcgha directory needed by BASE_DIR package installation setup");	
      COMMON::print_subaction("for $node_name_template ...") ;
      #
      # Install packages remotely
      #
      if ($answers_file eq "") {
	COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$package_subdir -M -R $node_root $package_list" ) ||
	  exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
      } else {
	COMMON::remote_exec ( "$node_ip", "$RPKGADD -n -a /tmp/admin -r /tmp/answers -d /mnt/$package_subdir -M -R $node_root $package_list" ) ||
	  exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
      }
    }
  } 

  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ||
    exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  
  #
  # remove the answers file from the remote node
  #
  if ($answers_file ne "") {
    COMMON::remote_exec ( "$node_ip", "$RM /tmp/answers" ) ||
      exit_install_pkg($admin_file, $answers_file, $node_ip, $package_list, $root_dir, $package_subdir);
  }

  #
  # remove previously created pkgadd admin and answers file
  #
  unlink($admin_file);
  if ($answers_file ne "") {
    unlink($answers_file);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) ||
    COMMON::error ( "Unable to NFS umount /mnt" );	
}

#==============================================================================
#
#    PATCH MANAGEMENT ROUTINES
#
#==============================================================================

#------------------------------------------------------------------------------
#
# Create the prerequisites for patching shared packages
#
#------------------------------------------------------------------------------

sub shared_patch_preparation {
  my $dir  = "var/sadm/system/admin" ;
  my $file = "INST_RELEASE" ;

  my $node_ip   = COMMON::get_external_ip($NODE_ID);

  COMMON::print_action("Creating prerequisites for patching shared packages ...") ;

  #
  # Remotely create directory
  #
  COMMON::remote_exec( $node_ip, "$RMKDIR -p $DIR_SUNWCGHA_EXPORTED/services/$dir" ) ||
    COMMON::error( "Unable to create $DIR_SUNWCGHA_EXPORTED/services/$dir" );

  #
  # Remotely copy file
  #
  COMMON::remote_exec($node_ip, "$RCOPY /$dir/$file $DIR_SUNWCGHA_EXPORTED/services/$dir/$file") ||
    COMMON::error( "Unable to copy $file on $DIR_SUNWCGHA_EXPORTED/services/$dir/$file" );
}

#------------------------------------------------------------------------------
#
#  clean up when a patch installation fails
#
#  1: node ip address
#
#------------------------------------------------------------------------------

sub exit_install_patch {

  my ($node_ip, $patch_id, $dir, $subdir) = @_;

  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" );
  COMMON::error("Unable to install patch $patch_id (from $dir/$subdir)" );
}

#------------------------------------------------------------------------------
#
# remove patch on a MEN
# 
#------------------------------------------------------------------------------

sub remove_local_patch {

  my ($node_id, $patch, $scope) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $optcmd ;
  my $adj ;

  my $patchid = $patch;
  if (substr($patch,0,1) eq "T") {
    # it's a temporary patch, remove the T to have the patch ID
    $patchid = substr($patch,1,length($patch)-1);
  }

  if ($scope eq $SCOPE_LOCAL) {
    $optcmd = "" ;
    $adj = "local" ;
  } elsif ($scope eq $SCOPE_SHARED) {
    $optcmd = "-R $SHARED_FS" ;
    $adj = "shared" ;
  } else {
    COMMON::error("Invalid option ($scope) for MEN patch installation") ;
  }
  COMMON::print_action("Removing $adj patch: $patch ...") ;	
		
  #
  # Remove patch remotely
  #
  my @list = split(" ",$patch) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
    COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM $optcmd $list[$i]" ) ;
  }
}

#------------------------------------------------------------------------------
#
# install patch on a MEN
# 
#------------------------------------------------------------------------------

sub install_local_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $scope) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $optcmd ;
  my $adj ;

  if ($scope eq $SCOPE_LOCAL) {
    $optcmd = "" ;
    $adj = "local" ;
  } elsif ($scope eq $SCOPE_SHARED) {
    $optcmd = "-R $SHARED_FS" ;
    $adj = "shared" ;
  } else {
    COMMON::error("Invalid option ($scope) for MEN patch installation") ;
  }

  COMMON::print_action("Installing $adj patch: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #   
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;
		
  #
  # Add patch remotely
  #
  COMMON::remote_exec ( "$node_ip", "$RPATCHADD -M /mnt/$patch_subdir $optcmd $patch" ) ||
    exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
      
  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# remove a patch for all DISKLESS on a MEN
# 
#------------------------------------------------------------------------------

sub remove_diskless_patch {

  my ($node_id, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Removing patch for diskless nodes: $patch ...") ;
		
  if (($where != $SOLARIS_LOC) && ($where != $DISKLESS_LOC)) {
    COMMON::error("internal error: install_diskless_patch location: $where (SOLARIS or DISKLESS required)");
  }
  
  my $patchid = $patch;
  if (substr($patch,0,1) eq "T") {
    # it's a temporary patch, remove the T to have the patch ID
    $patchid = substr($patch,1,length($patch)-1);
  }

  if ($where == $SOLARIS_LOC) {
    #
    # Remove patch from Solaris environment remotely
    #
    COMMON::print_subaction("for $DISKLESS_TARGET_OS service ...") ;
    my @list = split(" ",$patch) ;
    for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
      COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM -S $DISKLESS_TARGET_OS $list[$i]" ) ;
    }
  } else {

    #
    # Remove patches from diskless nodes remotely
    #
    for my $node (0..$#NODE_LIST) {
      if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	
	my @list = split(" ",$patch) ;
	for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
	 COMMON::remote_exec_ignore ( "$node_ip", "$RPATCHRM -R $node_root $list[$i]" ) ;
	}
      }
    }
  }
}

#------------------------------------------------------------------------------
#
# install patch for all diskless on a MEN
# 
#------------------------------------------------------------------------------

sub install_diskless_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Installing patch for diskless nodes: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;

  if (($where != $SOLARIS_LOC) && ($where != $DISKLESS_LOC)) {
    COMMON::error("internal error: install_diskless_patch location: $where (SOLARIS or DISKLESS required)");
  }

  if ($where == $SOLARIS_LOC) {

    #
    # Add patch on Solaris environment remotely
    #
    COMMON::print_subaction("for $DISKLESS_TARGET_OS service ...") ;
    if ($USE_DCSS) {
      COMMON::remote_exec ( "$node_ip", "$RNETRAOS patch -i -f -M /mnt/$patch_subdir -P $patch $DISKLESS_TARGET_OS" ) ||
	exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
    } else {
      COMMON::remote_exec ( "$node_ip", "$RPATCHADD -M /mnt/$patch_subdir -S $DISKLESS_TARGET_OS $patch" ) ||
	exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
    }

  } else {

    #
    # Install patches on diskless nodes remotely
    #
    for my $node (0..$#NODE_LIST) {
      if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	
	COMMON::remote_exec ( "$node_ip", "$RPATCHADD -R $node_root -M /mnt/$patch_subdir $patch" ) ||
	  exit_install_patch($node_ip, $patch, $root_dir, $patch_subdir);
      }
    }
  }
   
  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#==============================================================================
#
#    PATCH MANAGEMENT ROUTINES USING SMOSSERVICE
#
#==============================================================================

#------------------------------------------------------------------------------
#
# add a list of patches to the spool directory
#
#------------------------------------------------------------------------------

sub install_sm_diskless_patch {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Adding patch to the spool directory: $patch ...") ;

  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;

  #
  # add the patches to the spool area
  #
  my @list = split(" ",$patch) ;
  for (my $i = 0; $i < (scalar(@list)) ; $i++) {
    COMMON::remote_exec("$node_ip", 
			"$RSMOSSERVICE patch -p $PASSWORD -- " .
			"-a /mnt/$patch_subdir/$list[$i]" ) 
      || exit_install_patch($node_ip, $list[$i], $root_dir, $patch_subdir);
    SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $TRUE);
  }

  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# remove a list of patch from the spool directory
#
#------------------------------------------------------------------------------

sub remove_sm_diskless_patch {

  my ($node_id, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
   
  COMMON::print_action("Removing patch from the spool directory: $patch ...") ;


  #
  # remove the patches from the spool area
  #
  my @list = split(" ",$patch) ;
  for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {

    COMMON::remote_exec_ignore("$node_ip", 
			       "$RSMOSSERVICE patch -p $PASSWORD -- " .
			       "-r $list[$i]" ) 
  }
  SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $FALSE);
}

#------------------------------------------------------------------------------
#
# synchronize the patch 
#
#------------------------------------------------------------------------------

sub sync_sm_diskless_patch {

  my ($node_id) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);

  # can be TRUE, FALSE or missing (meaning FALSE)
  if (SEQUENCER::get_data("AT_LEAST_ONE_SM_PATCH") ne $TRUE) {
    return;
  }
   
  COMMON::print_action("Synchronizing spooled patch and updating diskless ...");
  #
  # updating all the diskless
  #
  COMMON::remote_exec("$node_ip", 
		      "$RSMOSSERVICE patch -p $PASSWORD -- -m -U" ) 
    || COMMON::error("Unable to update diskless with patches" );

  SEQUENCER::set_data("AT_LEAST_ONE_SM_PATCH", $FALSE);
}

#==============================================================================
#
#    PATCH MANAGEMENT ROUTINES USING PKGADD FOR EACH IMPACTED PACKAGE
#
#==============================================================================

#------------------------------------------------------------------------------
#
# retrieve the list of packages impacted by a patch
#
#------------------------------------------------------------------------------

sub retrieve_patch_pkg_list {

  my ($root_dir, $patch_subdir, $patch, $package_list) = @_;

  $$package_list = "";

  my @pkginfo_list = qx{find $root_dir/$patch_subdir/$patch -name pkginfo};
  foreach my $line  (@pkginfo_list) {
    my $end = index($line, "/pkginfo");
    my $start = rindex($line, "/", $end - 1) + 1;
    my $pkg = substr($line, $start, $end - $start);
    $$package_list = join(" ", $$package_list, $pkg);
  }
}

#------------------------------------------------------------------------------
#
# install a patch with pkgadd
#
#------------------------------------------------------------------------------

sub install_diskless_patch_pkgadd {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where, $location) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  my $package_list;
  my $basedir;
  my $dir;
  my $optcmd;
  my $adj;
   
  #
  # Test the product directory tree
  #
  COMMON::control_dir_is_readable ("$root_dir/$patch_subdir");

  #
  # mount ROOT_DIR
  #
  COMMON::remote_mount($node_ip, $root_dir, "/mnt") ;

  #
  # determine the package list
  #
  retrieve_patch_pkg_list($root_dir, $patch_subdir, $patch, \$package_list);


  COMMON::print_action("Adding patch with pkgadd: $patch (errors are ignored)...") ;


  if ($where == $SOLARIS_LOC) {
    set_package_info($location, \$dir, \$basedir, \$optcmd, \$adj);
  }

  # 
  # Create a pkgadd admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

  print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=nocheck
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;

  if ($where == $SOLARIS_LOC) {
    #
    # install packages on the common /usr
    #
    foreach my $pkg (split(' ',$package_list)) {
      COMMON::print_subaction("Installing package $pkg ...") ;
      COMMON::remote_exec_ignore ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$patch_subdir/$patch $optcmd $pkg" );
    }
    
  } else {

    #
    # Install packages on diskless concerned by the installation
    #
    foreach my $node (0..$#NODE_LIST) {
      if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	#
	# Install packages remotely
	#
	foreach my $pkg (split(' ',$package_list)) {
	  COMMON::print_subsubaction("Installing package $pkg ...") ;
	  COMMON::remote_exec_ignore ( "$node_ip", "$RPKGADD -n -a /tmp/admin -d /mnt/$patch)_subdir/$patch -M -R $node_root $pkg" );
	}
      }
    } 
  }
  
  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ||
    exit_install_patch( $node_ip, $patch, $root_dir, $patch_subdir);  
  
  #
  # remove previously created pkgadd admin file
  #
  unlink($admin_file);
  
  #
  # umount ROOT_DIR
  #   
  COMMON::remote_exec ( "$node_ip", "$RUMOUNT -f /mnt" ) || 
    COMMON::error ( "Unable to NFS umount /mnt" );
}

#------------------------------------------------------------------------------
#
# remove a patch installed with pkgadd
#
#------------------------------------------------------------------------------

sub remove_diskless_patch_pkgadd {

  my ($node_id, $root_dir, $patch_subdir, $patch, $where) = @_;

  my $node_ip     = COMMON::get_external_ip($node_id);
  my $admin_file  = "$WORKING_DIR/admin";
  my $package_list;
  my $basedir;
  my $dir;
  my $optcmd;
  my $adj;
  my $loc;


  ##### DO NOTHING: pkgrm will removed the package, not the patch !!!!
  return ;
  #####

  #
  # determine the package list
  #
  retrieve_patch_pkg_list($root_dir, $patch_subdir, $patch, \$package_list);


  COMMON::print_action("Removing patch with pkgrm: $patch ($package_list) ...");

  $basedir = "default" ;
  $dir     = "/opt/SUNWcgha" ;
  $optcmd  = "" ;
  $adj     = "local" ;
  $loc = "";

  if ($where == $SOLARIS_LOC) {
    $loc =  $SERVICE_USR_PATH;
    $basedir = $SERVICE_USR_BASEDIR;
    $optcmd  = "-M -R $loc" ;
    $adj     = "usr-specific" ;
    
  }

  # 
  # Create a pkgrm admin file
  #
  open(ADMIN, ">$admin_file") ||
    COMMON::error("Unable to create $admin_file");

   print ADMIN <<"FEO";
	
mail=
instance=overwrite
partial=nocheck
runlevel=quit
idepend=quit
rdepend=nocheck
space=quit
setuid=nocheck
conflict=nocheck
action=nocheck
basedir=$basedir

FEO


  close(ADMIN);
  
  #
  # copy the admin file on the remote node
  #
  COMMON::exec_cmd( "$RCP $admin_file $node_ip:/tmp/admin" ) ||
    COMMON::error ( "Unable to copy $admin_file on $node_ip:/tmp/admin") ;

  if ($where == $SOLARIS_LOC) {
    #
    # remove the packages in the reverse order
    #
    # do not display error
    my @list = split(" ",$package_list) ;
    for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
      # remove the ending part of package name like SUNWefcx.u
      # (the package is known as SUNWefcx after installation)
      my $pkg = $list[$i];
      my $pos = index($pkg, ".");
      if ($pos > -1) {
	$pkg = substr($pkg, 0, $pos);
      }
      COMMON::remote_exec_ignore( $node_ip, "echo y | $RPKGRM $optcmd -a /tmp/admin $pkg" ) ;
    }

  } else {
      
    #
    # remove packages for all diskless concerned by the installation
    #
    for my $node (0..$#NODE_LIST) {
      
      if (($NODE_LIST[$node]{type} == $DISKLESS_NODE) && ($NODE_LIST[$node]{concerned})) {
	
	my $node_name_template = COMMON::build_node_name($NODE_LIST[$node]{id}, "");
	my $node_root = "/export/root/$node_name_template";
	
	COMMON::print_subaction("for $node_name_template ...") ;
	#
	# remove the packages in the reverse order
	#
	# do not display error
	my @list = split(" ",$package_list) ;
	for (my $i = (scalar(@list) - 1) ; $i >= 0 ; $i--) {
	  # remove the ending part of package name like SUNWefcx.u
	  # (the package is known as SUNWefcx after installation)
	  my $pkg = $list[$i];
	  my $pos = index($pkg, ".");
	  if ($pos > -1) {
	    $pkg = substr($pkg, 0, $pos);
	  }
	  COMMON::remote_exec_ignore ( "$node_ip", "echo y | $RPKGRM -a /tmp/admin -R $node_root $pkg" ) ;
	}
      }
    }
  }

  #
  # remove the admin file from the remote node
  #
  COMMON::remote_exec ( "$node_ip", "$RM /tmp/admin" ) ;
  
  # remove previously created pkgadd admin file
  #
  unlink($admin_file);	
}

#==============================================================================
#
# INSTALLATION OF PACKAGES AND PATCHES BASED ON A COMPONENT DESCRIPTION
#
#==============================================================================

#------------------------------------------------------------------------------
#
# get the component reference
#
#------------------------------------------------------------------------------

sub get_component_idx {

  my ($component_array, $component) = @_;

  # as a specific OS product file can supersede common description
  # process the array in the reverse order

  for (my $i = (@{$component_array} - 1) ; $i >= 0 ; $i--) {
    if ($$component_array[$i]{name} eq $component) {
      return $i;
    }
  }

  COMMON::error("COMPONENT $component unknown");
}

#------------------------------------------------------------------------------
#
# get the location reference
#
#------------------------------------------------------------------------------

sub get_location_idx {

  my ($location_array, $location) = @_;

  for (my $i = 0 ; $i < @{$location_array} ; $i++) {
    if ($$location_array[$i]{name} eq $location) {
      return $i;
    }
  }

  COMMON::error("LOCATION $location unknown");
}

#------------------------------------------------------------------------------
#
# get the distrib root for master_eligible nodes
#
#------------------------------------------------------------------------------

sub get_men_distrib_root {

  my ($location) = @_;

  if ($location eq "NHAS") {
    return $NHAS_DIR;
  } elsif ($location eq "SOLARIS") {
    return $MEN_SOLARIS_DIR;
  }
}

#------------------------------------------------------------------------------
#
# get the distrib root for dataless nodes
#
#------------------------------------------------------------------------------

sub get_dataless_distrib_root {

  my ($location) = @_;

  if ($location eq "NHAS") {
    return $NHAS_DIR;
  } elsif ($location eq "SOLARIS") {
    return $DATALESS_SOLARIS_DIR;
  }
}

#------------------------------------------------------------------------------
#
# get the distrib root for diskless nodes
#
#------------------------------------------------------------------------------

sub get_diskless_distrib_root {

  my ($location) = @_;

  if ($location eq "NHAS") {
    return $NHAS_DIR;
  } elsif ($location eq "SOLARIS") {
    return $DISKLESS_SOLARIS_DIR;
  }
}

#------------------------------------------------------------------------------
#
# return component information for master-eligible node
#
#------------------------------------------------------------------------------

sub get_info_men_component {

  my ($component, $info) = @_;

  my $idx = get_component_idx(\@MEN_COMPONENT, $component);

  if ($info eq "DIR") {
    return $MEN_COMPONENT[$idx]{dir};
  } elsif ($info eq "PACKAGE") {
    return $MEN_COMPONENT[$idx]{pkg_men};
  } else {
    COMMON::error("INTERNAL: invalid info $info for component $component");
  }
    
}

#------------------------------------------------------------------------------
#
# return component information for dataless node
#
#------------------------------------------------------------------------------

sub get_info_dataless_component {

  my ($component, $info) = @_;

  my $idx = get_component_idx(\@DATALESS_COMPONENT, $component);

  if ($info eq "DIR") {
    return $DATALESS_COMPONENT[$idx]{dir};
  } elsif ($info eq "PACKAGE") {
    return $DATALESS_COMPONENT[$idx]{pkg_dataless};
  } else {
    COMMON::error("INTERNAL: invalid info $info for component $component");
  }
    
}

#------------------------------------------------------------------------------
#
# installation of a component on a master-eligible node
#
#------------------------------------------------------------------------------

sub install_men_component {

  my ($node_id, $component, $answer_file) = @_;

  my $idx = get_component_idx(\@MEN_COMPONENT, $component);
  my $loc = get_location_idx(\@MEN_LOCATION, $MEN_COMPONENT[$idx]{location});

  my $scope = $MEN_COMPONENT[$idx]{scope};

  my $root = get_men_distrib_root($MEN_COMPONENT[$idx]{location});

  if ($MEN_COMPONENT[$idx]{pkg_men} ne "") {
    install_local_pkg($node_id, 
		    $root, 
		    $MEN_LOCATION[$loc]{package}, 
		    $MEN_COMPONENT[$idx]{pkg_men}, 
		    $scope, 
		    $answer_file);
  }

  if ($MEN_COMPONENT[$idx]{patch_men} ne "") {
    install_local_patch($node_id, 
		      $root,
		      $MEN_LOCATION[$loc]{patch} , 
		      $MEN_COMPONENT[$idx]{patch_men},
		      $scope) ;
  }
}

#------------------------------------------------------------------------------
#
# remove a component from a master-eligible node
#
#------------------------------------------------------------------------------

sub remove_men_component {

  my ($node_id, $component) = @_;

  my $idx = get_component_idx(\@MEN_COMPONENT, $component);

  my $scope = $MEN_COMPONENT[$idx]{scope};

  if ($MEN_COMPONENT[$idx]{patch_men} ne "") {
    remove_local_patch($node_id, $MEN_COMPONENT[$idx]{patch_men}, $scope) ;
  }

  if ($MEN_COMPONENT[$idx]{pkg_men} ne "") {
    remove_local_pkg($node_id, $MEN_COMPONENT[$idx]{pkg_men}, $scope);
  }

}

#------------------------------------------------------------------------------
#
# installation of a component on a dataless node
#
#------------------------------------------------------------------------------

sub install_dataless_component {

  my ($node_id, $component, $answer_file) = @_;

  my $idx = get_component_idx(\@DATALESS_COMPONENT, $component);
  my $loc = get_location_idx(\@DATALESS_LOCATION,
			     $DATALESS_COMPONENT[$idx]{location});

  my $scope = $DATALESS_COMPONENT[$idx]{scope};

  my $root = get_dataless_distrib_root($DATALESS_COMPONENT[$idx]{location});

  if ($DATALESS_COMPONENT[$idx]{pkg_dataless} ne "") {
    install_local_pkg($node_id, 
		    $root, 
		    $DATALESS_LOCATION[$loc]{package}, 
		    $DATALESS_COMPONENT[$idx]{pkg_dataless}, 
		    $scope, 
		    $answer_file);
  }

  if ($DATALESS_COMPONENT[$idx]{patch_dataless} ne "") {
    install_local_patch($node_id, 
		      $root,
		      $DATALESS_LOCATION[$loc]{patch} , 
		      $DATALESS_COMPONENT[$idx]{patch_dataless},
		      $scope) ;
  }
}

#------------------------------------------------------------------------------
#
# remove a component from a dataless node
#
#------------------------------------------------------------------------------

sub remove_dataless_component {

  my ($node_id, $component) = @_;

  my $idx = get_component_idx(\@DATALESS_COMPONENT, $component);

  my $scope = $DATALESS_COMPONENT[$idx]{scope};

  if ($DATALESS_COMPONENT[$idx]{patch_dataless} ne "") {
    remove_local_patch($node_id, 
		       $DATALESS_COMPONENT[$idx]{patch_dataless}, $scope) ;
  }

  if ($DATALESS_COMPONENT[$idx]{pkg_dataless} ne "") {
    remove_local_pkg($node_id, $DATALESS_COMPONENT[$idx]{pkg_dataless},
		     $scope);
  }

}

#------------------------------------------------------------------------------
#
# installation of a diskless component
#
#------------------------------------------------------------------------------

sub install_diskless_component {

  my ($node_id, $component, $answer_file) = @_;

  my $idx = get_component_idx(\@DISKLESS_COMPONENT, $component);
  my $loc = get_location_idx(\@DISKLESS_LOCATION, 
			     $DISKLESS_COMPONENT[$idx]{location});

  my $scope = $DISKLESS_COMPONENT[$idx]{scope};
  if ($scope eq $SCOPE_SHARED) {
    return;
  }

  my $root = get_diskless_distrib_root($DISKLESS_COMPONENT[$idx]{location});

  #
  # install on the Solaris services (usr_specific)
  # during a full cluster install
  #
  if ($INSTALL_TYPE == $CLUSTER_INSTALL) {

    if ($DISKLESS_COMPONENT[$idx]{pkg_usr} ne "") {
      install_local_pkg($node_id, 
		      $root,
		      $DISKLESS_LOCATION[$loc]{package},
		      $DISKLESS_COMPONENT[$idx]{pkg_usr}, 
		      $SCOPE_USR_SPECIFIC, 
		      $answer_file);
    }
    
    if ($DISKLESS_COMPONENT[$idx]{patch_usr} ne "") {
      install_diskless_patch($node_id, 
			$root,
			$DISKLESS_LOCATION[$loc]{patch} , 
			$DISKLESS_COMPONENT[$idx]{patch_usr},
			$SOLARIS_LOC) ;
    }
  }

  #
  # install on the root of each diskless
  # 
  if ($DISKLESS_COMPONENT[$idx]{pkg_root} ne "") {
    install_diskless_pkg($node_id, 
		    $root,
		    $DISKLESS_LOCATION[$loc]{package},
		    $DISKLESS_COMPONENT[$idx]{pkg_root},
		    $answer_file);
  }
    
  if ($DISKLESS_COMPONENT[$idx]{patch_root} ne "") {
    install_diskless_patch($node_id, 
		       $root,
		       $DISKLESS_LOCATION[$loc]{patch} , 
		       $DISKLESS_COMPONENT[$idx]{patch_root},
		       $DISKLESS_LOC) ;
  }
}

#------------------------------------------------------------------------------
#
# remove a diskless component
#
#------------------------------------------------------------------------------

sub remove_diskless_component {

  my ($node_id, $component) = @_;

  my $idx = get_component_idx(\@DISKLESS_COMPONENT, $component);
  my $loc = get_location_idx(\@DISKLESS_LOCATION,
			     $DISKLESS_COMPONENT[$idx]{location});

  my $scope = $DISKLESS_COMPONENT[$idx]{scope};
  if ($scope eq $SCOPE_SHARED) {
    return;
  }

  my $root = get_diskless_distrib_root($DISKLESS_COMPONENT[$idx]{location});

  #
  # remove from the root of each diskless
  # 
  if ($DISKLESS_COMPONENT[$idx]{patch_root} ne "") {
    remove_diskless_patch($node_id, 
		      $DISKLESS_COMPONENT[$idx]{patch_root},
		      $DISKLESS_LOC) ;
  }

  if ($DISKLESS_COMPONENT[$idx]{pkg_root} ne "") {
    remove_diskless_pkg($node_id, 
		    $DISKLESS_COMPONENT[$idx]{pkg_root});
  }

  #
  # remove from the Solaris services (usr_specific)
  # during a full cluster install
  #
  if ($INSTALL_TYPE == $CLUSTER_INSTALL) {

    if ($DISKLESS_COMPONENT[$idx]{patch_usr} ne "") {
      remove_diskless_patch($node_id, 
			$DISKLESS_COMPONENT[$idx]{patch_usr},
			$SOLARIS_LOC) ;
    }

    if ($DISKLESS_COMPONENT[$idx]{pkg_usr} ne "") {
      remove_local_pkg($node_id, 
		      $DISKLESS_COMPONENT[$idx]{pkg_usr}, 
		      $SCOPE_USR_SPECIFIC);
    }
  }    
}

#------------------------------------------------------------------------------
#
# OS RELEASE AND PRODUCT MANAGEMENT
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#
# extract the release info from a file
#
#------------------------------------------------------------------------------

sub get_release {

  my ($release_file) = @_;

  my $release = qx{$GREP "Solaris" $release_file};

  $release =~ s/^[\s\t]*//;
  $release =~ s/[\s\t]*$//;
  chomp($release);

  return $release;
}

#------------------------------------------------------------------------------
#
# is the release a valid release ?
#
#------------------------------------------------------------------------------

sub check_release {

  my ($release) = @_ ;

  for my $i (0..$#OS_LIST) {
    if ($OS_LIST[$i]{idstring} eq $release) {
      return $TRUE;
    }
  }
  
  return $FALSE;
}

#------------------------------------------------------------------------------
#
# get the release index
#
#------------------------------------------------------------------------------

sub get_release_idx {

  my ($release) = @_ ;

  for my $i (0..$#OS_LIST) {
    if ($OS_LIST[$i]{idstring} eq $release) {
      return $i;
    }
  }
}

#------------------------------------------------------------------------------
#
# initialize all the information related to the distribution
# for master-eligible nodes
#
#------------------------------------------------------------------------------

sub init_men_distrib {

  my $idx = get_location_idx(\@MEN_LOCATION, "SOLARIS");

  $MEN_SOLARIS_PACKAGE_SUBDIR = $MEN_LOCATION[$idx]{package};
  $MEN_SOLARIS_PATCH_SUBDIR   = $MEN_LOCATION[$idx]{patch};

  $idx = get_location_idx(\@MEN_LOCATION, "NHAS");
					       
  $MEN_NHAS_PACKAGE_SUBDIR    = $MEN_LOCATION[$idx]{package};
  $MEN_NHAS_PATCH_SUBDIR      = $MEN_LOCATION[$idx]{patch};
}

#------------------------------------------------------------------------------
#
# initialize all the information related to the distribution
# for dataless nodes
#
#------------------------------------------------------------------------------

sub init_dataless_distrib {

  my $idx = get_location_idx(\@DATALESS_LOCATION, "SOLARIS");

  $DATALESS_SOLARIS_PACKAGE_SUBDIR = $DATALESS_LOCATION[$idx]{package};
  $DATALESS_SOLARIS_PATCH_SUBDIR   = $DATALESS_LOCATION[$idx]{patch};

  $idx = get_location_idx(\@DATALESS_LOCATION, "NHAS");
					       
  $DATALESS_NHAS_PACKAGE_SUBDIR    = $DATALESS_LOCATION[$idx]{package};
  $DATALESS_NHAS_PATCH_SUBDIR      = $DATALESS_LOCATION[$idx]{patch};
}

#------------------------------------------------------------------------------
#
# initialize all the information related to the distribution
# for diskless nodes
#
#------------------------------------------------------------------------------

sub init_diskless_distrib {

  my $idx = get_location_idx(\@DISKLESS_LOCATION, "SOLARIS");

  $DISKLESS_SOLARIS_PACKAGE_SUBDIR = $DISKLESS_LOCATION[$idx]{package};
  $DISKLESS_SOLARIS_PATCH_SUBDIR   = $DISKLESS_LOCATION[$idx]{patch};

  $idx = get_location_idx(\@DISKLESS_LOCATION, "NHAS");
					       
  $DISKLESS_NHAS_PACKAGE_SUBDIR    = $DISKLESS_LOCATION[$idx]{package};
  $DISKLESS_NHAS_PATCH_SUBDIR      = $DISKLESS_LOCATION[$idx]{patch};

  # diskless environment definition for service, packages and patches
  if (! $USE_DCSS) {
    $SERVICE_USR_PATH            = "/export/$DISKLESS_TARGET_OS/usr_sparc.all";
    $SERVICE_USR_BASEDIR         = "default";
    $SERVICE_USR_SOLARIS_PATH    = "/export/$DISKLESS_TARGET_OS";
    $SERVICE_USR_SOLARIS_BASEDIR = "/usr_sparc.all";
    $SERVICE_CLONE_OPT_PATH      = "/export/root/clone/$DISKLESS_TARGET_OS/$ARCH";
    $SERVICE_CLONE_OPT_BASEDIR   = "/opt";
  } else {
    $SERVICE_USR_PATH            = "/export/exec/$DISKLESS_TARGET_OS/";
    $SERVICE_USR_BASEDIR         = "default";
    $SERVICE_USR_SOLARIS_PATH    = "/export/exec/$DISKLESS_TARGET_OS/";
    $SERVICE_USR_SOLARIS_BASEDIR = "default";
  }
}

{
}
